# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import pods


POD1 = {'id': 123,
        'uuid': '66666666-7777-8888-9999-000000000000',
        'name': 'pod1',
        'desc': "desc",
        'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
        'images': ['image1', 'image2'],
        'labels': {'foo': 'bar'},
        'status': 'Running'
        }
POD2 = {'id': 124,
        'uuid': '66666666-7777-8888-9999-000000000001',
        'name': 'pod1',
        'desc': "desc",
        'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a53',
        'images': ['image1', 'image2'],
        'labels': {'foo': 'bar'},
        'status': 'Running'
        }
CREATE_POD = {'manifest_url': 'file:///a/b.json'}
UPDATED_POD = copy.deepcopy(POD1)
NEW_DESCR = 'new-description'
UPDATED_POD['description'] = NEW_DESCR

fake_responses = {
    '/v1/pods':
    {
        'GET': (
            {},
            {'pods': [POD1, POD2]},
        ),
        'POST': (
            {},
            CREATE_POD,
        ),
    },
    '/v1/pods/?bay_ident=%s' % (POD1['bay_uuid']):
    {
        'GET': (
            {},
            {'pods': [POD1, POD2]},
        ),
        'POST': (
            {},
            CREATE_POD,
        ),
    },
    '/v1/pods/%s/?bay_ident=%s' % (POD1['id'], POD1['bay_uuid']):
    {
        'GET': (
            {},
            POD1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_POD,
        ),
    },
    '/v1/pods/%s/?bay_ident=%s' % (POD1['name'], POD1['bay_uuid']):
    {
        'GET': (
            {},
            POD1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_POD,
        ),
    },
}


class PodManagerTest(testtools.TestCase):

    def setUp(self):
        super(PodManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = pods.PodManager(self.api)

    def test_pod_list(self):
        pods = self.mgr.list(POD1['bay_uuid'])
        expect = [
            ('GET', '/v1/pods/?bay_ident=%s' % (POD1['bay_uuid']), {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(pods, matchers.HasLength(2))

    def test_pod_show_by_id(self):
        pod = self.mgr.get(POD1['id'], POD1['bay_uuid'])
        expect = [
            ('GET', '/v1/pods/%s/?bay_ident=%s' % (POD1['id'],
                                                   POD1['bay_uuid']), {},
             None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(POD1['name'], pod.name)
        self.assertEqual(POD1['bay_uuid'], pod.bay_uuid)
        self.assertEqual(POD1['desc'], pod.desc)

    def test_pod_show_by_name(self):
        pod = self.mgr.get(POD1['name'], POD1['bay_uuid'])
        expect = [
            ('GET', '/v1/pods/%s/?bay_ident=%s' % (POD1['name'],
                                                   POD1['bay_uuid']),
             {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(POD1['name'], pod.name)
        self.assertEqual(POD1['bay_uuid'], pod.bay_uuid)
        self.assertEqual(POD1['desc'], pod.desc)

    def test_pod_create(self):
        pod = self.mgr.create(**CREATE_POD)
        expect = [
            ('POST', '/v1/pods', {}, CREATE_POD),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(pod)

    def test_pod_create_fail(self):
        CREATE_POD_FAIL = copy.deepcopy(CREATE_POD)
        CREATE_POD_FAIL["wrong_key"] = "wrong"
        self.assertRaisesRegexp(exceptions.InvalidAttribute,
                                ("Key must be in %s" %
                                 ','.join(pods.CREATION_ATTRIBUTES)),
                                self.mgr.create, **CREATE_POD_FAIL)
        self.assertEqual([], self.api.calls)

    def test_pod_delete_by_id(self):
        pod = self.mgr.delete(POD1['id'], POD1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/pods/%s/?bay_ident=%s' % (POD1['id'],
                                                      POD1['bay_uuid']),
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(pod)

    def test_pod_delete_by_name(self):
        pod = self.mgr.delete(POD1['name'], POD1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/pods/%s/?bay_ident=%s' % (POD1['name'],
                                                      POD1['bay_uuid']),
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(pod)

    def test_pod_update(self):
        patch = {'op': 'replace',
                 'value': NEW_DESCR,
                 'path': '/description'}
        pod = self.mgr.update(id=POD1['id'],
                              bay_ident=POD1['bay_uuid'],
                              patch=patch)
        expect = [
            ('PATCH', '/v1/pods/%s/?bay_ident=%s' % (POD1['id'],
                                                     POD1['bay_uuid']),
             {}, patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_DESCR, pod.description)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient import exceptions


def _show_pod(pod):
    del pod._info['links']
    utils.print_dict(pod._info)


@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
def do_pod_list(cs, args):
    """Print a list of registered pods."""
    bay = cs.bays.get(args.bay)
    if bay.status not in ['CREATE_COMPLETE', 'UPDATE_COMPLETE']:
        raise exceptions.InvalidAttribute(
            'Bay status for %s is: %s. We can not list pods in there until'
            ' the status is CREATE_COMPLETE or UPDATE_COMPLETE.' %
            (bay.uuid, bay.status))
    pods = cs.pods.list(args.bay)
    columns = ('uuid', 'name')
    utils.print_list(pods, columns,
                     {'versions': magnum_utils.print_list_field('versions')})


@utils.arg('--manifest-url',
           metavar='<manifest-url>',
           help='Name/URL of the pod file to use for creating PODs.')
@utils.arg('--manifest',
           metavar='<manifest>',
           help='File path of the pod file to use for creating PODs.')
@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help='ID or name of the bay.')
def do_pod_create(cs, args):
    """Create a pod."""
    bay = cs.bays.get(args.bay)
    if bay.status not in ['CREATE_COMPLETE', 'UPDATE_COMPLETE']:
        raise exceptions.InvalidAttribute(
            'Bay status for %s is: %s. We cannot create a %s'
            ' until the status is CREATE_COMPLETE or UPDATE_COMPLETE.' %
            (bay.uuid, bay.status, "pod"))
    opts = {}
    opts['manifest_url'] = args.manifest_url
    opts['bay_uuid'] = bay.uuid

    if args.manifest is not None and os.path.isfile(args.manifest):
        with open(args.manifest, 'r') as f:
            opts['manifest'] = f.read()

    node = cs.pods.create(**opts)
    _show_pod(node)
    pass


@utils.arg('pod', metavar='<pod-id>', help="UUID or name of pod")
@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help="Operations: 'add', 'replace' or 'remove'")
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help="Attributes to add/replace or remove "
         "(only PATH is necessary on remove)")
def do_pod_update(cs, args):
    """Update information about the given pod."""
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])
    p = patch[0]
    if p['path'] == '/manifest' and os.path.isfile(p['value']):
        with open(p['value'], 'r') as f:
            p['value'] = f.read()

    pod = cs.pods.update(args.pod, args.bay, patch)
    _show_pod(pod)


@utils.arg('pods',
           metavar='<pods>', nargs='+',
           help='ID or name of the (pod)s to delete.')
@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
def do_pod_delete(cs, args):
    """Delete specified pod."""
    for pod in args.pods:
        try:
            cs.pods.delete(pod, args.bay)
        except Exception as e:
            print("Delete for pod %(pod)s failed: %(e)s" %
                  {'pod': pod, 'e': e})
    pass


@utils.arg('pod',
           metavar='<pod>',
           help='ID or name of the pod to show.')
@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
def do_pod_show(cs, args):
    """Show details about the given pod."""
    pod = cs.pods.get(args.pod, args.bay)
    _show_pod(pod)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from magnumclient import exceptions
from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.bays.BayManager.list')
    def test_bay_list_success(self, mock_list):
        self._test_arg_success('bay-list')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.bays.BayManager.list')
    def test_bay_list_failure(self, mock_list):
        self._test_arg_failure('bay-list --wrong',
                               self._unrecognized_arg_error)
        self.assertFalse(mock_list.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.get')
    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_success(self, mock_create, mock_get):
        self._test_arg_success('bay-create --name test --baymodel xxx '
                               '--node-count 123 --timeout 15')
        self.assertTrue(mock_create.called)

        self._test_arg_success('bay-create --baymodel xxx')
        self.assertTrue(mock_create.called)

        self._test_arg_success('bay-create --name test --baymodel xxx')
        self.assertTrue(mock_create.called)

        self._test_arg_success('bay-create --baymodel xxx --node-count 123')
        self.assertTrue(mock_create.called)

        self._test_arg_success('bay-create --baymodel xxx --node-count 123 '
                               '--master-count 123')
        self.assertTrue(mock_create.called)

        self._test_arg_success('bay-create --baymodel xxx '
                               '--timeout 15')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.get')
    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_success_only_baymodel_arg(self, mock_create, mock_get):
        self._test_arg_success('bay-create --baymodel xxx')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_only_name(self, mock_create):
        self._test_arg_failure('bay-create --name test',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_only_node_count(self, mock_create):
        self._test_arg_failure('bay-create --node-count 1',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_invalid_node_count(self, mock_create):
        self._test_arg_failure('bay-create --baymodel xxx --node-count test',
                               self._invalid_value_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_only_bay_create_timeout(self, mock_create):
        self._test_arg_failure('bay-create --timeout 15',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_no_arg(self, mock_create):
        self._test_arg_failure('bay-create',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.create')
    def test_bay_create_failure_invalid_master_count(self, mock_create):
        self._test_arg_failure('bay-create --baymodel xxx --master-count test',
                               self._invalid_value_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.delete')
    def test_bay_delete_success(self, mock_delete):
        self._test_arg_success('bay-delete xxx')
        self.assertTrue(mock_delete.called)
        self.assertEqual(1, mock_delete.call_count)

    @mock.patch('magnumclient.v1.bays.BayManager.delete')
    def test_bay_delete_multiple_id_success(self, mock_delete):
        self._test_arg_success('bay-delete xxx xyz')
        self.assertTrue(mock_delete.called)
        self.assertEqual(2, mock_delete.call_count)

    @mock.patch('magnumclient.v1.bays.BayManager.delete')
    def test_bay_delete_failure_no_arg(self, mock_delete):
        self._test_arg_failure('bay-delete', self._few_argument_error)
        self.assertFalse(mock_delete.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    def test_bay_show_success(self, mock_show):
        self._test_arg_success('bay-show xxx')
        self.assertTrue(mock_show.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    def test_bay_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('bay-show', self._few_argument_error)
        self.assertFalse(mock_show.called)

    @mock.patch('magnumclient.v1.bays.BayManager.update')
    def test_bay_update_success(self, mock_update):
        self._test_arg_success('bay-update test add test=test')
        self.assertTrue(mock_update.called)

    @mock.patch('magnumclient.v1.bays.BayManager.update')
    def test_bay_update_success_many_attribute(self, mock_update):
        self._test_arg_success('bay-update test add test=test test1=test1')
        self.assertTrue(mock_update.called)

    @mock.patch('magnumclient.v1.bays.BayManager.update')
    def test_bay_update_failure_wrong_op(self, mock_update):
        _error_msg = [
            '.*?^usage: magnum bay-update ',
            '.*?^error: argument <op>: invalid choice: ',
            ".*?^Try 'magnum help bay-update' for more information."
            ]
        self._test_arg_failure('bay-update test wrong test=test', _error_msg)
        self.assertFalse(mock_update.called)

    @mock.patch('magnumclient.v1.bays.BayManager.update')
    def test_bay_update_failure_wrong_attribute(self, mock_update):
        _error_msg = [
            '.*?^ERROR: Attributes must be a list of PATH=VALUE'
            ]
        self.assertRaises(exceptions.CommandError, self._test_arg_failure,
                          'bay-update test add test', _error_msg)
        self.assertFalse(mock_update.called)

    @mock.patch('magnumclient.v1.bays.BayManager.update')
    def test_bay_update_failure_few_args(self, mock_update):
        _error_msg = [
            '.*?^usage: magnum bay-update ',
            '.*?^error: (the following arguments|too few arguments)',
            ".*?^Try 'magnum help bay-update' for more information."
            ]
        self._test_arg_failure('bay-update', _error_msg)
        self.assertFalse(mock_update.called)

        self._test_arg_failure('bay-update test', _error_msg)
        self.assertFalse(mock_update.called)

        self._test_arg_failure('bay-update test add', _error_msg)
        self.assertFalse(mock_update.called)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from magnumclient.tests.v1 import shell_test_base
from magnumclient.v1 import certificates_shell


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.get')
    def test_ca_show_success(self, mock_cert_get, mock_bay_get):
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_COMPLETE"
        mock_bay_get.return_value = mockbay
        self._test_arg_success('ca-show '
                               '--bay xxx')
        self.assertTrue(mock_cert_get.called)

    @mock.patch('os.path.isfile')
    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.create')
    def test_ca_sign_success(
        self, mock_cert_create, mock_bay_get, mock_isfile):
        mock_isfile.return_value = True
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_COMPLETE"
        mock_bay_get.return_value = mockbay

        fake_csr = 'fake-csr'
        file_mock = mock.mock_open(read_data=fake_csr)
        with mock.patch.object(certificates_shell, 'open', file_mock):
            self._test_arg_success('ca-sign '
                                   '--csr path/csr.pem '
                                   '--bay xxx')
            self.assertTrue(mock_cert_create.called)

    @mock.patch('os.path.isfile')
    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.certificates.CertificateManager.create')
    def test_ca_sign_with_not_csr(
        self, mock_cert_create, mock_bay_get, mock_isfile):
        mock_isfile.return_value = False
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_COMPLETE"
        mock_bay_get.return_value = mockbay

        fake_csr = 'fake-csr'
        file_mock = mock.mock_open(read_data=fake_csr)
        with mock.patch.object(certificates_shell, 'open', file_mock):
            self._test_arg_success('ca-sign '
                                   '--csr path/csr.pem '
                                   '--bay xxx')
            mock_isfile.assert_called_once_with('path/csr.pem')
            self.assertFalse(file_mock.called)
            self.assertFalse(mock_cert_create.called)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path

from magnumclient.common import cliutils as utils


def _show_cert(certificate):
    print(certificate.pem)


@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help='ID or name of the bay.')
def do_ca_show(cs, args):
    """Show details about the CA certificate for a bay."""
    bay = cs.bays.get(args.bay)
    opts = {
        'bay_uuid': bay.uuid
    }

    cert = cs.certificates.get(**opts)
    _show_cert(cert)


@utils.arg('--csr',
           metavar='<csr>',
           help='File path of the csr file to send to Magnum to get signed.')
@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help='ID or name of the bay.')
def do_ca_sign(cs, args):
    """Generate the CA certificate for a bay."""
    bay = cs.bays.get(args.bay)
    opts = {
        'bay_uuid': bay.uuid
    }

    if args.csr is None or not os.path.isfile(args.csr):
        print('A CSR must be provided.')
        return

    with open(args.csr, 'r') as f:
        opts['csr'] = f.read()

    cert = cs.certificates.create(**opts)
    _show_cert(cert)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils


def _show_coe_service(service):
    utils.print_dict(service._info)


@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help="UUID or Name of Bay")
def do_coe_service_list(cs, args):
    """Print a list of coe services."""
    services = cs.services.list(args.bay)
    columns = ('uuid', 'name', 'bay_uuid')
    utils.print_list(services, columns,
                     {'versions': magnum_utils.print_list_field('versions')})


@utils.arg('--manifest-url',
           metavar='<manifest-url>',
           help='Name/URL of the service file to use for creating services.')
@utils.arg('--manifest',
           metavar='<manifest>',
           help='File path of the service file to use for creating services.')
@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help='Id or name of the bay.')
def do_coe_service_create(cs, args):
    """Create a coe service."""
    bay = cs.bays.get(args.bay)
    if bay.status not in ['CREATE_COMPLETE', 'UPDATE_COMPLETE']:
        print('Bay status for %s is: %s. We can not create a service in bay '
              'until the status is CREATE_COMPLETE or UPDATE_COMPLETE.' %
              (args.bay, bay.status))
        return

    opts = {}
    opts['manifest_url'] = args.manifest_url
    opts['bay_uuid'] = bay.uuid

    if args.manifest is not None and os.path.isfile(args.manifest):
        with open(args.manifest, 'r') as f:
            opts['manifest'] = f.read()

    service = cs.services.create(**opts)
    _show_coe_service(service)


@utils.arg('services', metavar='<services>', help="UUID or name of service")
@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help="Operations: 'add', 'replace' or 'remove'")
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help="Attributes to add/replace or remove "
         "(only PATH is necessary on remove)")
def do_coe_service_update(cs, args):
    """Update information about the given coe service."""
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])
    p = patch[0]
    if p['path'] == '/manifest' and os.path.isfile(p['value']):
        with open(p['value'], 'r') as f:
            p['value'] = f.read()

    service = cs.services.update(args.services, args.bay, patch)
    _show_coe_service(service)


@utils.arg('services', metavar='<services>', nargs='+',
           help='ID or name of the service to delete.')
@utils.arg('--bay', required=True, metavar='<bay>',
           help="UUID or Name of Bay")
def do_coe_service_delete(cs, args):
    """Delete specified coe service."""
    for service in args.services:
        try:
            cs.services.delete(service, args.bay)
        except Exception as e:
            print("Delete for service %(service)s failed: %(e)s" %
                  {'service': service, 'e': e})


@utils.arg('services', metavar='<services>',
           help='ID or name of the service to show.')
@utils.arg('--bay', required=True,
           metavar='<bay>', help="UUID or Name of Bay")
def do_coe_service_show(cs, args):
    """Show details about the given coe service."""
    service = cs.services.get(args.services, args.bay)
    _show_coe_service(service)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from magnumclient import exceptions
from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.list')
    def test_rc_list_success(self, mock_list, mock_get):
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_COMPLETE"
        mock_get.return_value = mockbay
        self._test_arg_success('rc-list --bay bay_ident')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.list')
    def test_rc_list_failure(self, mock_list):
        self._test_arg_failure('rc-list --bay bay_ident --wrong',
                               self._unrecognized_arg_error)
        self.assertFalse(mock_list.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.list')
    def test_rc_list_failure_invalid_bay_status(self, mock_list, mock_get):
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_IN_PROGRESS"
        mock_get.return_value = mockbay
        self.assertRaises(exceptions.InvalidAttribute, self._test_arg_failure,
                          'rc-list --bay bay_ident', self._bay_status_error)
        self.assertFalse(mock_list.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.create')
    def test_rc_create_success(self, mock_create, mock_get):
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_COMPLETE"
        mock_get.return_value = mockbay

        self._test_arg_success('rc-create '
                               '--bay xxx '
                               '--manifest test '
                               '--manifest-url test_url')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.create')
    def test_rc_create_failure_few_arg(self, mock_create, mock_get):
        self._test_arg_failure('rc-create '
                               '--manifest test '
                               '--manifest-url test_url',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('rc-create '
                               'bay xxx '
                               '--manifest-url test_url',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.create')
    def test_rc_create_failure_invalid_bay_status(self, mock_create, mock_get):
        mockbay = mock.MagicMock()
        mockbay.status = "CREATE_IN_PROGRESS"
        mock_get.return_value = mockbay
        self.assertRaises(exceptions.InvalidAttribute, self._test_arg_failure,
                          'rc-create '
                          '--bay xxx '
                          '--manifest test '
                          '--manifest-url test_url',
                          self._bay_status_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.delete')
    def test_rc_delete_success(self, mock_delete):
        self._test_arg_success('rc-delete xxx --bay zzz')
        self.assertTrue(mock_delete.called)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.delete')
    def test_rc_delete_failure_no_arg(self, mock_delete):
        self._test_arg_failure('rc-delete xxx', self._mandatory_arg_error)
        self.assertFalse(mock_delete.called)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.get')
    def test_rc_show_success(self, mock_show):
        self._test_arg_success('rc-show xxx --bay zzz')
        self.assertTrue(mock_show.called)
        self.assertEqual(1, mock_show.call_count)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.get')
    def test_rc_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('rc-show xxx', self._mandatory_arg_error)
        self.assertFalse(mock_show.called)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.update')
    def test_rc_update_success(self, mock_update):
        self._test_arg_success('rc-update xxx --bay zzz replace xxx=xxx')
        self.assertTrue(mock_update.called)
        self.assertEqual(1, mock_update.call_count)

    @mock.patch('magnumclient.v1.replicationcontrollers.'
                'ReplicationControllerManager.update')
    def test_rc_update_failure_no_arg(self, mock_update):
        self._test_arg_failure('rc-update xxx', self._few_argument_error)
        self.assertFalse(mock_update.called)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os.path

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils


def _show_baymodel(baymodel):
    del baymodel._info['links']
    utils.print_dict(baymodel._info)


@utils.arg('--name',
           metavar='<name>',
           help='Name of the baymodel to create.')
@utils.arg('--image-id',
           required=True,
           metavar='<image-id>',
           help='The name or UUID of the base image to customize for the bay.')
@utils.arg('--keypair-id',
           required=True,
           metavar='<keypair-id>',
           help='The name or UUID of the SSH keypair to load into the'
           ' Bay nodes.')
@utils.arg('--external-network-id',
           required=True,
           metavar='<external-network-id>',
           help='The external Neutron network ID to connect to this bay'
           ' model.')
@utils.arg('--coe',
           required=True,
           metavar='<coe>',
           help='Specify the Container Orchestration Engine to use.')
@utils.arg('--fixed-network',
           metavar='<fixed-network>',
           help='The private Neutron network name to connect to this bay'
           ' model.')
@utils.arg('--network-driver',
           metavar='<network-driver>',
           help='The network driver name for instantiating container'
           ' networks.')
@utils.arg('--volume-driver',
           metavar='<volume-driver>',
           help='The volume driver name for instantiating container'
           ' volume.')
@utils.arg('--dns-nameserver',
           metavar='<dns-nameserver>',
           default='8.8.8.8',
           help='The DNS nameserver to use for this Bay.')
@utils.arg('--flavor-id',
           metavar='<flavor-id>',
           default='m1.medium',
           help='The nova flavor id to use when launching the bay.')
@utils.arg('--master-flavor-id',
           metavar='<master-flavor-id>',
           help='The nova flavor id to use when launching the master node '
           'of the bay.')
@utils.arg('--docker-volume-size',
           metavar='<docker-volume-size>',
           type=int,
           help='Specify the number of size in GB '
                'for the docker volume to use.')
@utils.arg('--http-proxy',
           metavar='<http-proxy>',
           help='The http_proxy address to use for nodes in bay.')
@utils.arg('--https-proxy',
           metavar='<https-proxy>',
           help='The https_proxy address to use for nodes in bay.')
@utils.arg('--no-proxy',
           metavar='<no-proxy>',
           help='The no_proxy address to use for nodes in bay.')
@utils.arg('--labels', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
           action='append', default=[],
           help='Arbitrary labels in the form of key=value pairs '
                'to associate with a baymodel. '
                'May be used multiple times.')
@utils.arg('--tls-disabled',
           action='store_true', default=False,
           help='Disable TLS in the Bay.')
@utils.arg('--public',
           action='store_true', default=False,
           help='Make baymodel public.')
@utils.arg('--registry-enabled',
           action='store_true', default=False,
           help='Enable docker registry in the Bay')
def do_baymodel_create(cs, args):
    """Create a baymodel."""
    opts = {}
    opts['name'] = args.name
    opts['flavor_id'] = args.flavor_id
    opts['master_flavor_id'] = args.master_flavor_id
    opts['image_id'] = args.image_id
    opts['keypair_id'] = args.keypair_id
    opts['external_network_id'] = args.external_network_id
    opts['fixed_network'] = args.fixed_network
    opts['network_driver'] = args.network_driver
    opts['volume_driver'] = args.volume_driver
    opts['dns_nameserver'] = args.dns_nameserver
    opts['docker_volume_size'] = args.docker_volume_size
    opts['coe'] = args.coe
    opts['http_proxy'] = args.http_proxy
    opts['https_proxy'] = args.https_proxy
    opts['no_proxy'] = args.no_proxy
    opts['labels'] = magnum_utils.format_labels(args.labels)
    opts['tls_disabled'] = args.tls_disabled
    opts['public'] = args.public
    opts['registry_enabled'] = args.registry_enabled

    baymodel = cs.baymodels.create(**opts)
    _show_baymodel(baymodel)


@utils.arg('baymodels',
           metavar='<baymodels>',
           nargs='+',
           help='ID or name of the (baymodel)s to delete.')
def do_baymodel_delete(cs, args):
    """Delete specified baymodel."""
    for baymodel in args.baymodels:
        try:
            cs.baymodels.delete(baymodel)
        except Exception as e:
            print("Delete for baymodel %(baymodel)s failed: %(e)s" %
                  {'baymodel': baymodel, 'e': e})


@utils.arg('baymodel',
           metavar='<baymodel>',
           help='ID of the baymodel to show.')
def do_baymodel_show(cs, args):
    """Show details about the given baymodel."""
    baymodel = cs.baymodels.get(args.baymodel)
    _show_baymodel(baymodel)


@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help='Maximum number of baymodels to return')
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help='Column to sort results by')
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help='Direction to sort. "asc" or "desc".')
def do_baymodel_list(cs, args):
    """Print a list of bay models."""
    nodes = cs.baymodels.list(limit=args.limit,
                              sort_key=args.sort_key,
                              sort_dir=args.sort_dir)
    columns = ('uuid', 'name')
    utils.print_list(nodes, columns,
                     {'versions': magnum_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.arg('baymodel', metavar='<baymodel>', help="UUID or name of baymodel")
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help="Operations: 'add', 'replace' or 'remove'")
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help="Attributes to add/replace or remove "
         "(only PATH is necessary on remove)")
def do_baymodel_update(cs, args):
    """Updates one or more baymodel attributes."""
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])
    p = patch[0]
    if p['path'] == '/manifest' and os.path.isfile(p['value']):
        with open(p['value'], 'r') as f:
            p['value'] = f.read()

    baymodel = cs.baymodels.update(args.baymodel, patch)
    _show_baymodel(baymodel)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils


def _show_bay(bay):
    del bay._info['links']
    utils.print_dict(bay._info)


def do_bay_list(cs, args):
    """Print a list of available bays."""
    bays = cs.bays.list()
    columns = ('uuid', 'name', 'node_count', 'master_count', 'status')
    utils.print_list(bays, columns,
                     {'versions': magnum_utils.print_list_field('versions')})


@utils.arg('--name',
           metavar='<name>',
           help='Name of the bay to create.')
@utils.arg('--baymodel',
           required=True,
           metavar='<baymodel>',
           help='ID or name of the baymodel.')
@utils.arg('--node-count',
           metavar='<node-count>',
           type=int,
           default=1,
           help='The bay node count.')
@utils.arg('--master-count',
           metavar='<master-count>',
           type=int,
           default=1,
           help='The number of master nodes for the bay.')
@utils.arg('--discovery-url',
           metavar='<discovery-url>',
           help='Specifies custom discovery url for node discovery.')
@utils.arg('--timeout',
           metavar='<timeout>',
           type=int,
           default=0,
           help='The timeout for bay creation in minutes. Set '
                'to 0 for no timeout. The default is no timeout.')
def do_bay_create(cs, args):
    """Create a bay."""
    baymodel = cs.baymodels.get(args.baymodel)

    opts = {}
    opts['name'] = args.name
    opts['baymodel_id'] = baymodel.uuid
    opts['node_count'] = args.node_count
    opts['master_count'] = args.master_count
    opts['discovery_url'] = args.discovery_url
    opts['bay_create_timeout'] = args.timeout
    try:
        bay = cs.bays.create(**opts)
        _show_bay(bay)
    except Exception as e:
        print("Create for bay %s failed: %s" %
              (opts['name'], e))


@utils.arg('bay',
           metavar='<bay>',
           nargs='+',
           help='ID or name of the (bay)s to delete.')
def do_bay_delete(cs, args):
    """Delete specified bay."""
    for id in args.bay:
        try:
            cs.bays.delete(id)
        except Exception as e:
            print("Delete for bay %(bay)s failed: %(e)s" %
                  {'bay': id, 'e': e})


@utils.arg('bay',
           metavar='<bay>',
           help='ID or name of the bay to show.')
def do_bay_show(cs, args):
    """Show details about the given bay."""
    bay = cs.bays.get(args.bay)
    _show_bay(bay)


@utils.arg('bay', metavar='<bay>', help="UUID or name of bay")
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help="Operations: 'add', 'replace' or 'remove'")
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help="Attributes to add/replace or remove "
         "(only PATH is necessary on remove)")
def do_bay_update(cs, args):
    """Update information about the given bay."""
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])
    bay = cs.bays.update(args.bay, patch)
    _show_bay(bay)



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import replicationcontrollers as rcs


RC1 = {'id': 123,
       'uuid': '66666666-7777-8888-9999-000000000001',
       'name': 'rc1',
       'images': ['image1'],
       'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a51',
       'selector': {'name': 'bar1'},
       'replicas': 1
       }
RC2 = {'id': 124,
       'uuid': '66666666-7777-8888-9999-000000000002',
       'name': 'rc2',
       'images': ['image2'],
       'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
       'selector': {'name': 'bar2'},
       'replicas': 2
       }
CREATE_RC = {'manifest_url': 'file:///a/b.json'}
UPDATED_RC = copy.deepcopy(RC1)
NEW_REPLICAS = 3
UPDATED_RC['replicas'] = NEW_REPLICAS

fake_responses = {
    '/v1/rcs':
    {
        'GET': (
            {},
            {'rcs': [RC1, RC2]},
        ),
        'POST': (
            {},
            CREATE_RC,
        ),
    },
    '/v1/rcs/?bay_ident=%s' % (RC1['bay_uuid']):
    {
        'GET': (
            {},
            {'rcs': [RC1, RC2]},
        ),
    },
    '/v1/rcs/%s/?bay_ident=%s' % (RC1['id'], RC1['bay_uuid']):
    {
        'GET': (
            {},
            RC1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_RC,
        ),
    },
    '/v1/rcs/%s/?bay_ident=%s' % (RC1['name'], RC1['bay_uuid']):
    {
        'GET': (
            {},
            RC1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_RC,
        ),
    },
}


class RCManagerTest(testtools.TestCase):

    def setUp(self):
        super(RCManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = rcs.ReplicationControllerManager(self.api)

    def test_rc_list(self):
        rcs = self.mgr.list(RC1['bay_uuid'])
        expect = [
            ('GET', '/v1/rcs/?bay_ident=%s' % (RC1['bay_uuid']), {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(rcs, matchers.HasLength(2))

    def test_rc_show_by_id(self):
        rc = self.mgr.get(RC1['id'], RC1['bay_uuid'])
        expect = [
            ('GET', '/v1/rcs/%s/?bay_ident=%s' % (RC1['id'],
                                                  RC1['bay_uuid']), {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(RC1['name'], rc.name)
        self.assertEqual(RC1['bay_uuid'], rc.bay_uuid)
        self.assertEqual(RC1['replicas'], rc.replicas)

    def test_rc_show_by_name(self):
        rc = self.mgr.get(RC1['name'], RC1['bay_uuid'])
        expect = [
            ('GET', '/v1/rcs/%s/?bay_ident=%s' % (RC1['name'],
                                                  RC1['bay_uuid']), {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(RC1['name'], rc.name)
        self.assertEqual(RC1['bay_uuid'], rc.bay_uuid)
        self.assertEqual(RC1['replicas'], rc.replicas)

    def test_rc_create(self):
        rc = self.mgr.create(**CREATE_RC)
        expect = [
            ('POST', '/v1/rcs', {}, CREATE_RC),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(rc)

    def test_rc_create_fail(self):
        CREATE_RC_FAIL = copy.deepcopy(CREATE_RC)
        CREATE_RC_FAIL["wrong_key"] = "wrong"
        self.assertRaisesRegexp(exceptions.InvalidAttribute,
                                ("Key must be in %s" %
                                 ','.join(rcs.CREATION_ATTRIBUTES)),
                                self.mgr.create, **CREATE_RC_FAIL)
        self.assertEqual([], self.api.calls)

    def test_rc_delete_by_id(self):
        rc = self.mgr.delete(RC1['id'], RC1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/rcs/%s/?bay_ident=%s' % (RC1['id'],
                                                     RC1['bay_uuid']),
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(rc)

    def test_rc_delete_by_name(self):
        rc = self.mgr.delete(RC1['name'], RC1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/rcs/%s/?bay_ident=%s' % (RC1['name'],
                                                     RC1['bay_uuid']),
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(rc)

    def test_rc_update(self):
        patch = {'op': 'replace',
                 'value': NEW_REPLICAS,
                 'path': '/replicas'}
        rc = self.mgr.update(id=RC1['id'],
                             bay_ident=RC1['bay_uuid'],
                             patch=patch)
        expect = [
            ('PATCH', '/v1/rcs/%s/?bay_ident=%s' % (RC1['id'],
                                                    RC1['bay_uuid']), {},
             patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_REPLICAS, rc.replicas)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import loading
from keystoneauth1 import session as ksa_session

from magnumclient.common import httpclient
from magnumclient.v1 import baymodels
from magnumclient.v1 import bays
from magnumclient.v1 import certificates
from magnumclient.v1 import containers
from magnumclient.v1 import mservices
from magnumclient.v1 import pods
from magnumclient.v1 import replicationcontrollers as rcs
from magnumclient.v1 import services


class Client(object):
    def __init__(self, username=None, api_key=None, project_id=None,
                 project_name=None, auth_url=None, magnum_url=None,
                 endpoint_type=None, service_type='container',
                 region_name=None, input_auth_token=None,
                 session=None, password=None, auth_type='password',
                 interface='public', service_name=None, insecure=False,
                 user_domain_id=None, user_domain_name=None,
                 project_domain_id=None, project_domain_name=None):

        # We have to keep the api_key are for backwards compat, but let's
        # remove it from the rest of our code since it's not a keystone
        # concept
        if not password:
            password = api_key
        # Backwards compat for people assing in endpoint_type
        if endpoint_type:
            interface = endpoint_type

        if magnum_url and input_auth_token:
            auth_type = 'admin_token'
            session = None
            loader_kwargs = dict(
                token=input_auth_token,
                endpoint=magnum_url)
        elif input_auth_token and not session:
            auth_type = 'token'
            loader_kwargs = dict(
                token=input_auth_token,
                auth_url=auth_url,
                project_id=project_id,
                project_name=project_name,
                user_domain_id=user_domain_id,
                user_domain_name=user_domain_name,
                project_domain_id=project_domain_id,
                project_domain_name=project_domain_name)
        else:
            loader_kwargs = dict(
                username=username,
                password=password,
                auth_url=auth_url,
                project_id=project_id,
                project_name=project_name,
                user_domain_id=user_domain_id,
                user_domain_name=user_domain_name,
                project_domain_id=project_domain_id,
                project_domain_name=project_domain_name)

        # Backwards compatibility for people not passing in Session
        if session is None:
            loader = loading.get_plugin_loader(auth_type)

            # This should be able to handle v2 and v3 Keystone Auth
            auth_plugin = loader.load_from_options(**loader_kwargs)
            session = ksa_session.Session(
                auth=auth_plugin, verify=(not insecure))

        client_kwargs = {}
        if magnum_url:
            client_kwargs['endpoint_override'] = magnum_url

        if not magnum_url:
            try:
                # Trigger an auth error so that we can throw the exception
                # we always have
                session.get_endpoint(
                    service_type=service_type,
                    service_name=service_name,
                    interface=interface,
                    region_name=region_name)
            except Exception:
                raise RuntimeError("Not Authorized")

        self.http_client = httpclient.SessionClient(
            service_type=service_type,
            service_name=service_name,
            interface=interface,
            region_name=region_name,
            session=session,
            **client_kwargs)
        self.bays = bays.BayManager(self.http_client)
        self.certificates = certificates.CertificateManager(self.http_client)
        self.baymodels = baymodels.BayModelManager(self.http_client)
        self.containers = containers.ContainerManager(self.http_client)
        self.pods = pods.PodManager(self.http_client)
        self.rcs = rcs.ReplicationControllerManager(self.http_client)
        self.services = services.ServiceManager(self.http_client)
        self.mservices = mservices.MServiceManager(self.http_client)



# -*- coding: utf-8 -*-
#
# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from magnumclient import exceptions as exc
from magnumclient.i18n import _


def common_filters(marker=None, limit=None, sort_key=None, sort_dir=None):
    """Generate common filters for any list request.

    :param marker: entity ID from which to start returning entities.
    :param limit: maximum number of entities to return.
    :param sort_key: field to use for sorting.
    :param sort_dir: direction of sorting: 'asc' or 'desc'.
    :returns: list of string filters.
    """
    filters = []
    if isinstance(limit, int) and limit > 0:
        filters.append('limit=%s' % limit)
    if marker is not None:
        filters.append('marker=%s' % marker)
    if sort_key is not None:
        filters.append('sort_key=%s' % sort_key)
    if sort_dir is not None:
        filters.append('sort_dir=%s' % sort_dir)
    return filters


def split_and_deserialize(string):
    """Split and try to JSON deserialize a string.

    Gets a string with the KEY=VALUE format, split it (using '=' as the
    separator) and try to JSON deserialize the VALUE.
    :returns: A tuple of (key, value).
    """
    try:
        key, value = string.split("=", 1)
    except ValueError:
        raise exc.CommandError(_('Attributes must be a list of '
                                 'PATH=VALUE not "%s"') % string)
    try:
        value = json.loads(value)
    except ValueError:
        pass

    return (key, value)


def args_array_to_patch(op, attributes):
    patch = []
    for attr in attributes:
        # Sanitize
        if not attr.startswith('/'):
            attr = '/' + attr
        if op in ['add', 'replace']:
            path, value = split_and_deserialize(attr)
            patch.append({'op': op, 'path': path, 'value': value})

        elif op == "remove":
            # For remove only the key is needed
            patch.append({'op': op, 'path': attr})
        else:
            raise exc.CommandError(_('Unknown PATCH operation: %s') % op)
    return patch


def format_labels(lbls, parse_comma=True):
    '''Reformat labels into dict of format expected by the API.'''

    if not lbls:
        return {}

    if parse_comma:
        # expect multiple invocations of --labels but fall back
        # to , delimited if only one --labels is specified
        if len(lbls) == 1:
            lbls = lbls[0].split(',')

    labels = {}
    for l in lbls:
        try:
            (k, v) = l.split(('='), 1)
        except ValueError:
            raise exc.CommandError(_('labels must be a list of KEY=VALUE '
                                     'not %s') % l)
        if k not in labels:
            labels[k] = v
        else:
            if not isinstance(labels[k], list):
                labels[k] = [labels[k]]
            labels[k].append(v)

    return labels


def print_list_field(field):
    return lambda obj: ', '.join(getattr(obj, field))



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import services


SERVICE1 = {'id': 123,
            'uuid': '66666666-7777-8888-9999-000000000001',
            'name': 'service1',
            'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a51',
            'selector': {'name': 'bar1'},
            'labels': {'name': 'foo1'},
            'ip': '10.0.0.3',
            'port': 8080
            }
SERVICE2 = {'id': 124,
            'uuid': '66666666-7777-8888-9999-000000000002',
            'name': 'service2',
            'bay_uuid': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
            'selector': {'name': 'bar2'},
            'labels': {'name': 'foo2'},
            'ip': '10.0.0.4',
            'port': 8081
            }
CREATE_SVC = {'manifest_url': 'file:///a/b.json'}
UPDATED_SVC = copy.deepcopy(SERVICE1)
NEW_SELECTOR = {'name': 'bar3'}
UPDATED_SVC['selector'] = NEW_SELECTOR

fake_responses = {
    '/v1/services':
    {
        'GET': (
            {},
            {'services': [SERVICE1, SERVICE2]},
        ),
        'POST': (
            {},
            CREATE_SVC,
        ),
    },
    '/v1/services/?bay_ident=%s' % (SERVICE1['bay_uuid']):
    {
        'GET': (
            {},
            {'services': [SERVICE1, SERVICE2]},
        ),
        'POST': (
            {},
            CREATE_SVC,
        ),
    },
    '/v1/services/%s/?bay_ident=%s' % (SERVICE1['id'],
                                       SERVICE1['bay_uuid']):
    {
        'GET': (
            {},
            SERVICE1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_SVC,
        ),
    },
    '/v1/services/%s/?bay_ident=%s' % (SERVICE1['name'],
                                       SERVICE1['bay_uuid']):
    {
        'GET': (
            {},
            SERVICE1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_SVC,
        ),
    },
}


class ServiceManagerTest(testtools.TestCase):

    def setUp(self):
        super(ServiceManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = services.ServiceManager(self.api)

    def test_coe_service_list(self):
        services = self.mgr.list(SERVICE1['bay_uuid'])
        expect = [
            ('GET', '/v1/services/?bay_ident=%s' % (SERVICE1['bay_uuid']),
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(services, matchers.HasLength(2))

    def test_coe_service_show_by_id(self):
        service = self.mgr.get(SERVICE1['id'], SERVICE1['bay_uuid'])
        expect = [
            ('GET', '/v1/services/%s/?bay_ident=%s' % (SERVICE1['id'],
                                                       SERVICE1['bay_uuid']),
             {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SERVICE1['name'], service.name)
        self.assertEqual(SERVICE1['bay_uuid'], service.bay_uuid)
        self.assertEqual(SERVICE1['ip'], service.ip)

    def test_coe_service_show_by_name(self):
        service = self.mgr.get(SERVICE1['name'], SERVICE1['bay_uuid'])
        expect = [
            ('GET', '/v1/services/%s/?bay_ident=%s' % (SERVICE1['name'],
                                                       SERVICE1['bay_uuid']),
             {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SERVICE1['name'], service.name)
        self.assertEqual(SERVICE1['bay_uuid'], service.bay_uuid)
        self.assertEqual(SERVICE1['ip'], service.ip)

    def test_coe_service_create(self):
        service = self.mgr.create(**CREATE_SVC)
        expect = [
            ('POST', '/v1/services', {}, CREATE_SVC),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(service)

    def test_coe_service_create_fail(self):
        CREATE_SVC_FAIL = copy.deepcopy(CREATE_SVC)
        CREATE_SVC_FAIL["wrong_key"] = "wrong"
        self.assertRaisesRegexp(exceptions.InvalidAttribute,
                                ("Key must be in %s" %
                                 ','.join(services.CREATION_ATTRIBUTES)),
                                self.mgr.create, **CREATE_SVC_FAIL)
        self.assertEqual([], self.api.calls)

    def test_coe_service_delete_by_id(self):
        service = self.mgr.delete(SERVICE1['id'], SERVICE1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/services/%s/?bay_ident=%s' % (
                SERVICE1['id'], SERVICE1['bay_uuid']), {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(service)

    def test_coe_service_delete_by_name(self):
        service = self.mgr.delete(SERVICE1['name'], SERVICE1['bay_uuid'])
        expect = [
            ('DELETE', '/v1/services/%s/?bay_ident=%s' % (
                SERVICE1['name'], SERVICE1['bay_uuid']), {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(service)

    def test_coe_service_update(self):
        patch = {'op': 'replace',
                 'value': NEW_SELECTOR,
                 'path': '/selector'}
        service = self.mgr.update(service_id=SERVICE1['id'],
                                  bay_ident=SERVICE1['bay_uuid'],
                                  patch=patch)
        expect = [
            ('PATCH', '/v1/services/%s/?bay_ident=%s' % (
                SERVICE1['id'], SERVICE1['bay_uuid']), {}, patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_SELECTOR, service.selector)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers

from magnumclient.tests import utils
from magnumclient.v1 import mservices


SERVICE1 = {'id': 123,
            'host': 'fake-host1',
            'binary': 'fake-bin1',
            'state': 'up',
            }
SERVICE2 = {'id': 124,
            'host': 'fake-host2',
            'binary': 'fake-bin2',
            'state': 'down',
            }

fake_responses = {
    '/v1/mservices':
    {
        'GET': (
            {},
            {'mservices': [SERVICE1, SERVICE2]},
        ),
    },
}


class MServiceManagerTest(testtools.TestCase):

    def setUp(self):
        super(MServiceManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = mservices.MServiceManager(self.api)

    def test_coe_service_list(self):
        mservices = self.mgr.list()
        expect = [
            ('GET', '/v1/mservices', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(mservices, matchers.HasLength(2))



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient import exceptions
from magnumclient.tests import utils
from magnumclient.v1 import containers


CONTAINER1 = {'id': 123,
              'uuid': '66666666-7777-8888-9999-000000000001',
              'bay_uuid': '25d5d872-1f4e-4134-ae15-c5fa38cb09a3',
              'name': 'container1',
              'image': 'c-image1',
              'command': 'c-command1',
              'memory': '512m',
              }
CONTAINER2 = {'id': 124,
              'uuid': '66666666-7777-8888-9999-000000000002',
              'bay_uuid': '25d5d872-1f4e-4134-ae15-c5fa38cb09a3',
              'name': 'container1',
              'image': 'c-image2',
              'command': 'c-command2',
              'memory': '2g',
              }

CREATE_CONTAINER = copy.deepcopy(CONTAINER1)
del CREATE_CONTAINER['id']
del CREATE_CONTAINER['uuid']

UPDATED_CONTAINER = copy.deepcopy(CONTAINER1)
NEW_NAME = 'newcontainer'
UPDATED_CONTAINER['name'] = NEW_NAME

fake_responses = {
    '/v1/containers':
    {
        'GET': (
            {},
            {'containers': [CONTAINER1, CONTAINER2]},
        ),
        'POST': (
            {},
            CREATE_CONTAINER,
        ),
    },
    '/v1/containers/?bay_ident=25d5d872-1f4e-4134-ae15-c5fa38cb09a3':
    {
        'GET': (
            {},
            {'containers': [CONTAINER1, CONTAINER2]},
        ),
        'POST': (
            {},
            CREATE_CONTAINER,
        ),
    },
    '/v1/containers/%s' % CONTAINER1['id']:
    {
        'GET': (
            {},
            CONTAINER1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_CONTAINER,
        ),
    },
    '/v1/containers/%s' % CONTAINER1['name']:
    {
        'GET': (
            {},
            CONTAINER1
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_CONTAINER,
        ),
    },
    '/v1/containers/%s/start' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/start' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/stop' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/stop' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/pause' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/pause' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/unpause' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/unpause' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/reboot' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/reboot' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            CONTAINER1,
        ),
    },
    '/v1/containers/%s/logs' % CONTAINER1['id']:
    {
        'GET': (
            {},
            {'output': 'login now'},
        ),
    },
    '/v1/containers/%s/logs' % CONTAINER1['name']:
    {
        'GET': (
            {},
            {'output': 'login now'},
        ),
    },
    '/v1/containers/%s/execute?command=ls' % CONTAINER1['id']:
    {
        'PUT': (
            {},
            {'output': '/home'},
        ),
    },
    '/v1/containers/%s/execute?command=ls' % CONTAINER1['name']:
    {
        'PUT': (
            {},
            {'output': '/home'},
        ),
    },
}


class ContainerManagerTest(testtools.TestCase):

    def setUp(self):
        super(ContainerManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = containers.ContainerManager(self.api)

    def test_container_list(self):
        containers = self.mgr.list()
        expect = [
            ('GET', '/v1/containers', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(containers, matchers.HasLength(2))

    def test_container_list_with_bay(self):
        containers = self.mgr.list(
            bay_ident='25d5d872-1f4e-4134-ae15-c5fa38cb09a3')
        expect = [
            ('GET', '/v1/containers/?bay_ident=25d5d872-1f4e-4134'
                    '-ae15-c5fa38cb09a3', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(containers, matchers.HasLength(2))

    def test_container_show(self):
        container = self.mgr.get(CONTAINER1['id'])
        expect = [
            ('GET', '/v1/containers/%s' % CONTAINER1['id'], {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(CONTAINER1['name'], container.name)
        self.assertEqual(CONTAINER1['image'], container.image)
        self.assertEqual(CONTAINER1['command'], container.command)
        self.assertEqual(CONTAINER1['memory'], container.memory)

    def test_container_create(self):
        container = self.mgr.create(**CREATE_CONTAINER)
        expect = [
            ('POST', '/v1/containers', {}, CREATE_CONTAINER),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_create_fail(self):
        CREATE_CONTAINER_FAIL = copy.deepcopy(CREATE_CONTAINER)
        CREATE_CONTAINER_FAIL["wrong_key"] = "wrong"
        self.assertRaisesRegexp(exceptions.InvalidAttribute,
                                ("Key must be in %s" %
                                 ','.join(containers.CREATION_ATTRIBUTES)),
                                self.mgr.create, **CREATE_CONTAINER_FAIL)
        self.assertEqual([], self.api.calls)

    def test_container_delete(self):
        container = self.mgr.delete(CONTAINER1['id'])
        expect = [
            ('DELETE', '/v1/containers/%s' % CONTAINER1['id'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(container)

    def test_container_update(self):
        patch = {'op': 'replace',
                 'value': NEW_NAME,
                 'path': '/name'}
        container = self.mgr.update(id=CONTAINER1['id'], patch=patch)
        expect = [
            ('PATCH', '/v1/containers/%s' % CONTAINER1['id'], {}, patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_NAME, container.name)

    def test_container_start(self):
        container = self.mgr.start(CONTAINER1['id'])
        expect = [
            ('PUT', '/v1/containers/%s/start' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_stop(self):
        container = self.mgr.stop(CONTAINER1['id'])
        expect = [
            ('PUT', '/v1/containers/%s/stop' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_reboot(self):
        container = self.mgr.reboot(CONTAINER1['id'])
        expect = [
            ('PUT', '/v1/containers/%s/reboot' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_pause(self):
        container = self.mgr.pause(CONTAINER1['id'])
        expect = [
            ('PUT', '/v1/containers/%s/pause' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_unpause(self):
        container = self.mgr.unpause(CONTAINER1['id'])
        expect = [
            ('PUT', '/v1/containers/%s/unpause' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(container)

    def test_container_logs(self):
        logs = self.mgr.logs(CONTAINER1['id'])
        expect = [
            ('GET', '/v1/containers/%s/logs' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual('login now', logs)

    def test_container_execute(self):
        output = self.mgr.execute(CONTAINER1['id'], 'ls')
        expect = [
            ('PUT', '/v1/containers/%s/execute?command=ls' % CONTAINER1['id'],
                    {'Content-Length': '0'}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual('/home', output)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from magnumclient import exceptions
from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.containers.ContainerManager.list')
    def test_container_list_success(self, mock_list):
        self._test_arg_success('container-list')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.list')
    def test_container_list_success_with_bay(self, mock_list):
        self._test_arg_success('container-list --bay bay_uuid')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.list')
    def test_container_list_failure(self, mock_list):
        self._test_arg_failure('container-list --wrong',
                               self._unrecognized_arg_error)
        self.assertFalse(mock_list.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.containers.ContainerManager.create')
    def test_container_create_success(self, mock_create, mock_bay_get):
        mock_bay = mock.MagicMock()
        mock_bay.status = "CREATE_COMPLETE"
        mock_bay_get.return_value = mock_bay
        self._test_arg_success('container-create '
                               '--image test-image '
                               '--bay test-bay')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.containers.ContainerManager.create')
    def test_container_create_failure_without_image(self, mock_create,
                                                    mock_bay_get):
        mock_bay = mock.MagicMock()
        mock_bay.status = "CREATE_COMPLETE"
        mock_bay_get.return_value = mock_bay
        self._test_arg_failure('container-create '
                               '--bay test-bay',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.bays.BayManager.get')
    @mock.patch('magnumclient.v1.containers.ContainerManager.create')
    def test_container_create_failure_invalid_bay_status(self, mock_create,
                                                         mock_bay_get):
        mock_bay = mock.MagicMock()
        mock_bay.status = "CREATE_IN_PROGRESS"
        mock_bay_get.return_value = mock_bay
        self.assertRaises(exceptions.InvalidAttribute, self._test_arg_failure,
                          'container-create '
                          '--image test-image '
                          '--bay test-bay',
                          self._bay_status_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.delete')
    def test_container_delete_success(self, mock_delete):
        self._test_arg_success('container-delete xxx')
        self.assertTrue(mock_delete.called)
        self.assertEqual(1, mock_delete.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.delete')
    def test_container_delete_multiple_id_success(self, mock_delete):
        self._test_arg_success('container-delete xxx xyz')
        self.assertTrue(mock_delete.called)
        self.assertEqual(2, mock_delete.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.delete')
    def test_container_delete_failure_no_arg(self, mock_delete):
        self._test_arg_failure('container-delete', self._few_argument_error)
        self.assertFalse(mock_delete.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.get')
    def test_container_show_success(self, mock_show):
        self._test_arg_success('container-show xxx')
        self.assertTrue(mock_show.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.get')
    def test_container_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('container-show', self._few_argument_error)
        self.assertFalse(mock_show.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.reboot')
    def test_container_reboot_success(self, mock_reboot):
        self._test_arg_success('container-reboot xxx')
        self.assertTrue(mock_reboot.called)
        self.assertEqual(1, mock_reboot.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.reboot')
    def test_container_reboot_multiple_id_success(self, mock_reboot):
        self._test_arg_success('container-reboot xxx xyz')
        self.assertTrue(mock_reboot.called)
        self.assertEqual(2, mock_reboot.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.reboot')
    def test_container_reboot_failure_no_arg(self, mock_reboot):
        self._test_arg_failure('container-reboot', self._few_argument_error)
        self.assertFalse(mock_reboot.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.stop')
    def test_container_stop_success(self, mock_stop):
        self._test_arg_success('container-stop xxx')
        self.assertTrue(mock_stop.called)
        self.assertEqual(1, mock_stop.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.stop')
    def test_container_stop_multiple_id_success(self, mock_stop):
        self._test_arg_success('container-stop xxx xyz')
        self.assertTrue(mock_stop.called)
        self.assertEqual(2, mock_stop.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.stop')
    def test_container_stop_failure_no_arg(self, mock_stop):
        self._test_arg_failure('container-stop', self._few_argument_error)
        self.assertFalse(mock_stop.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.start')
    def test_container_start_success(self, mock_start):
        self._test_arg_success('container-start xxx')
        self.assertTrue(mock_start.called)
        self.assertEqual(1, mock_start.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.start')
    def test_container_start_multiple_id_success(self, mock_start):
        self._test_arg_success('container-start xxx xyz')
        self.assertTrue(mock_start.called)
        self.assertEqual(2, mock_start.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.start')
    def test_container_start_failure_no_arg(self, mock_start):
        self._test_arg_failure('container-start', self._few_argument_error)
        self.assertFalse(mock_start.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.pause')
    def test_container_pause_success(self, mock_pause):
        self._test_arg_success('container-pause xxx')
        self.assertTrue(mock_pause.called)
        self.assertEqual(1, mock_pause.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.pause')
    def test_container_multiple_id_pause_success(self, mock_pause):
        self._test_arg_success('container-pause xxx xyz')
        self.assertTrue(mock_pause.called)
        self.assertEqual(2, mock_pause.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.pause')
    def test_container_pause_failure_no_arg(self, mock_pause):
        self._test_arg_failure('container-pause', self._few_argument_error)
        self.assertFalse(mock_pause.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.unpause')
    def test_container_unpause_success(self, mock_unpause):
        self._test_arg_success('container-unpause xxx')
        self.assertTrue(mock_unpause.called)
        self.assertEqual(1, mock_unpause.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.unpause')
    def test_container_unpause_multiple_id_success(self, mock_unpause):
        self._test_arg_success('container-unpause xxx xyz')
        self.assertTrue(mock_unpause.called)
        self.assertEqual(2, mock_unpause.call_count)

    @mock.patch('magnumclient.v1.containers.ContainerManager.unpause')
    def test_container_unpause_failure_no_arg(self, mock_unpause):
        self._test_arg_failure('container-unpause', self._few_argument_error)
        self.assertFalse(mock_unpause.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.logs')
    def test_container_logs_success(self, mock_logs):
        self._test_arg_success('container-logs xxx')
        self.assertTrue(mock_logs.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.logs')
    def test_container_logs_failure_no_arg(self, mock_logs):
        self._test_arg_failure('container-logs', self._few_argument_error)
        self.assertFalse(mock_logs.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.execute')
    def test_container_execute_success(self, mock_execute):
        self._test_arg_success('container-exec xxx '
                               '--command ls')
        self.assertTrue(mock_execute.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.execute')
    def test_container_execute_failure_no_option(self, mock_execute):
        self._test_arg_failure('container-exec xxx',
                               self._mandatory_arg_error)
        self.assertFalse(mock_execute.called)

        self._test_arg_failure('container-exec --command ls',
                               self._few_argument_error)
        self.assertFalse(mock_execute.called)

    @mock.patch('magnumclient.v1.containers.ContainerManager.execute')
    def test_container_execute_failure_no_arg(self, mock_execute):
        self._test_arg_failure('container-exec', self._few_argument_error)
        self.assertFalse(mock_execute.called)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from magnumclient.tests.v1 import shell_test_base


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--image-id test_image '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--coe swarm '
                               '--dns-nameserver test_dns '
                               '--flavor-id test_flavor '
                               '--fixed-network public '
                               '--volume-driver test_volume '
                               '--network-driver test_driver '
                               '--labels key=val '
                               '--master-flavor-id test_flavor '
                               '--docker-volume-size 10 '
                               '--public')
        self.assertTrue(mock_create.called)

        self._test_arg_success('baymodel-create '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe kubernetes '
                               '--name test ')

        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_success_with_registry_enabled(
        self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--network-driver test_driver '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm '
                               '--registry-enabled')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_public_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --network-driver test_driver '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm'
                               '--public')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_success_with_master_flavor(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--image-id test_image '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--coe swarm '
                               '--dns-nameserver test_dns '
                               '--master-flavor-id test_flavor')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_docker_vol_size_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --docker-volume-size 4514 '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm'
                               )
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_fixed_network_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --fixed-network private '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_network_driver_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --network-driver test_driver '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_volume_driver_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --volume-driver test_volume '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_http_proxy_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --fixed-network private '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm '
                               '--http-proxy http_proxy ')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_https_proxy_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --fixed-network private '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm '
                               '--https-proxy https_proxy ')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_no_proxy_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test --fixed-network private '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm '
                               '--no-proxy no_proxy ')

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_labels_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--labels key=val '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_separate_labels_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--labels key1=val1 '
                               '--labels key2=val2 '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_combined_labels_success(self, mock_create):
        self._test_arg_success('baymodel-create '
                               '--name test '
                               '--labels key1=val1,key2=val2 '
                               '--keypair-id test_keypair '
                               '--external-network-id test_net '
                               '--image-id test_image '
                               '--coe swarm')
        self.assertTrue(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.create')
    def test_baymodel_create_failure_few_arg(self, mock_create):
        self._test_arg_failure('baymodel-create '
                               '--name test', self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('baymodel-create '
                               '--image-id test', self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('baymodel-create '
                               '--keypair-id test', self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('baymodel-create '
                               '--external-network-id test',
                               self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('baymodel-create '
                               '--coe test', self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

        self._test_arg_failure('baymodel-create', self._mandatory_arg_error)
        self.assertFalse(mock_create.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.get')
    def test_baymodel_show_success(self, mock_show):
        self._test_arg_success('baymodel-show xxx')
        self.assertTrue(mock_show.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.get')
    def test_baymodel_show_failure_no_arg(self, mock_show):
        self._test_arg_failure('baymodel-show', self._few_argument_error)
        self.assertFalse(mock_show.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.delete')
    def test_baymodel_delete_success(self, mock_delete):
        self._test_arg_success('baymodel-delete xxx')
        self.assertTrue(mock_delete.called)
        self.assertEqual(1, mock_delete.call_count)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.delete')
    def test_baymodel_delete_multiple_id_success(self, mock_delete):
        self._test_arg_success('baymodel-delete xxx xyz')
        self.assertTrue(mock_delete.called)
        self.assertEqual(2, mock_delete.call_count)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.delete')
    def test_baymodel_delete_failure_no_arg(self, mock_delete):
        self._test_arg_failure('baymodel-delete', self._few_argument_error)
        self.assertFalse(mock_delete.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.list')
    def test_baymodel_list_success(self, mock_list):
        self._test_arg_success('baymodel-list')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.list')
    def test_baymodel_list_success_with_arg(self, mock_list):
        self._test_arg_success('baymodel-list '
                               '--limit 1 '
                               '--sort-dir asc '
                               '--sort-key uuid')
        self.assertTrue(mock_list.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.list')
    def test_baymodel_list_failure_invalid_arg(self, mock_list):
        _error_msg = [
            '.*?^usage: magnum baymodel-list ',
            '.*?^error: argument --sort-dir: invalid choice: ',
            ".*?^Try 'magnum help baymodel-list' for more information."
            ]
        self._test_arg_failure('baymodel-list --sort-dir aaa', _error_msg)
        self.assertFalse(mock_list.called)

    @mock.patch('magnumclient.v1.baymodels.BayModelManager.list')
    def test_baymodel_list_failure(self, mock_list):
        self._test_arg_failure('baymodel-list --wrong',
                               self._unrecognized_arg_error)
        self.assertFalse(mock_list.called)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient import exceptions


def _show_container(container):
    utils.print_dict(container._info)


@utils.arg('--name',
           metavar='<name>',
           help='name of the container')
@utils.arg('--image',
           required=True,
           metavar='<image>',
           help='name or ID of the image')
@utils.arg('--bay',
           required=True,
           metavar='<bay>',
           help='ID or name of the bay.')
@utils.arg('--command',
           metavar='<command>',
           help='Send command to the container')
@utils.arg('--memory',
           metavar='<memory>',
           help='The container memory size (format: <number><optional unit>, '
                'where unit = b, k, m or g)')
def do_container_create(cs, args):
    """Create a container."""
    bay = cs.bays.get(args.bay)
    if bay.status not in ['CREATE_COMPLETE',
                          'UPDATE_IN_PROGRESS', 'UPDATE_COMPLETE']:
        raise exceptions.InvalidAttribute(
            'Bay status for %s is: %s. We cannot create a %s'
            ' unless the status is CREATE_COMPLETE, UPDATE_IN_PROGRESS'
            ' or UPDATE_COMPLETE.' % (bay.uuid, bay.status, "container"))
        return
    opts = {}
    opts['name'] = args.name
    opts['image'] = args.image
    opts['bay_uuid'] = bay.uuid
    opts['command'] = args.command
    opts['memory'] = args.memory
    _show_container(cs.containers.create(**opts))


@utils.arg('--bay',
           metavar='<bay>', help="UUID or Name of Bay")
def do_container_list(cs, args):
    """Print a list of available containers."""
    opts = {}
    opts['bay_ident'] = args.bay
    containers = cs.containers.list(**opts)
    columns = ('uuid', 'name', 'status', 'bay_uuid')
    utils.print_list(containers, columns,
                     {'versions': magnum_utils.print_list_field('versions')})


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID or name of the (container)s to delete.')
def do_container_delete(cs, args):
    """Delete specified containers."""
    for container in args.containers:
        try:
            cs.containers.delete(container)
        except Exception as e:
            print("Delete for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('container',
           metavar='<container>',
           help='ID or name of the container to show.')
@utils.arg('--json',
           action='store_true',
           default=False,
           help='Print JSON representation of the container.')
def do_container_show(cs, args):
    """Show details of a container."""
    container = cs.containers.get(args.container)
    if args.json:
        print(json.dumps(container._info))
    else:
        _show_container(container)


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID or name of the (container)s to reboot.')
def do_container_reboot(cs, args):
    """Reboot specified containers."""
    for container in args.containers:
        try:
            cs.containers.reboot(container)
        except Exception as e:
            print("Reboot for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID or name of the (container)s to stop.')
def do_container_stop(cs, args):
    """Stop specified containers."""
    for container in args.containers:
        try:
            cs.containers.stop(container)
        except Exception as e:
            print("Stop for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID of the (container)s to start.')
def do_container_start(cs, args):
    """Start specified containers."""
    for container in args.containers:
        try:
            cs.containers.start(container)
        except Exception as e:
            print("Start for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID or name of the (container)s to pause.')
def do_container_pause(cs, args):
    """Pause specified containers."""
    for container in args.containers:
        try:
            cs.containers.pause(container)
        except Exception as e:
            print("Pause for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('containers',
           metavar='<container>',
           nargs='+',
           help='ID or name of the (container)s to unpause.')
def do_container_unpause(cs, args):
    """Unpause specified containers."""
    for container in args.containers:
        try:
            cs.containers.unpause(container)
        except Exception as e:
            print("Unpause for container %(container)s failed: %(e)s" %
                  {'container': container, 'e': e})


@utils.arg('container',
           metavar='<container>',
           help='ID or name of the container to get logs for.')
def do_container_logs(cs, args):
    """Get logs of a container."""
    logs = cs.containers.logs(args.container)
    print(logs)


@utils.arg('container',
           metavar='<container>',
           help='ID or name of the container to execute command in.')
@utils.arg('--command',
           required=True,
           metavar='<command>',
           help='The command to execute')
def do_container_exec(cs, args):
    """Execute command in a container."""
    output = cs.containers.execute(args.container, args.command)
    print(output)
